
//New Goertzel


/*The Goertzel Algorithm
http://www.embedded.com/design/configurable-systems/4024443/The-Goertzel-Algorithm

https://en.wikipedia.org/wiki/Goertzel_algorithm

http://www.numerix-dsp.com/goertzel.html

https://en.wikipedia.org/wiki/Goertzel_algorithm

*/

#include "stdafx.h"

#include <stdint.h>

#include <iostream>
#include <fstream>
#include <iomanip>

#define _USE_MATH_DEFINES
#include <math.h>

#include "PLs.h"

#include "pool.h"

/*
Create filter parameters
this will be part of a header generator to use with MSP430 source code
*/

#define SCALE_INT_S 10000
//We will fix the sample rate to 1000, the tone filter rolls off starting at 300Hz
#define SAMPLE_RATE_S 1000

typedef uint32_t TYPE_COEFF;


// .........................................................
struct PARAMS
{
	uint16_t freq; //the target frequency times 10
	uint16_t n_size; //samples in window, adjusted for target frequency
	int16_t coeff; //the filtering parameter
};

// .........................................................
void CreateParams( double freq, PARAMS& params )
{
	double size= (double)SAMPLE_RATE_S / freq;
	params.n_size= (uint16_t)( size * (int)( 220 / size ) );

	//for PL frequency precision is tenths of a hertz
	params.freq= (uint16_t)( freq * 10 );
	
	//temps to calc coeff
	double k= params.n_size * freq / SAMPLE_RATE_S;
	double omega= 2 * M_PI * k / params.n_size;

	//set coefficient
	double coeff= cos( omega ) * 2;
	params.coeff= (TYPE_COEFF)( coeff * SCALE_INT_S + .5 );
}

// .........................................................
double CalcPower( int32_t* testData, PARAMS& params, bool print= false )
{
	std::ofstream* file= NULL;
	if( print )
		file= new std::ofstream( "testout2.txt", std::ios::ate );

	double q0= 0;
	double q1= 0;
	double q2= 0;
	uint32_t range= 0;
	for( auto i= 0; i < params.n_size; ++i )
	{
		q0= ( (double)params.coeff / SCALE_INT_S ) * q1 - q2 + (double)testData[ i ];
		q2= q1;
		q1= q0;
		if( print )
			*file << i << " " << ( q2 ) << " " << q0 << std::endl;

		if( abs( q0 ) > range )
			range= (uint32_t)abs( q0 );
	}

	if( print )
	{
		file->close( );
		delete file;
	}

	//double power= q1 * q1 + q2 * q2 - q1 * q2;
	double power= (double)range / 1000;
	return power;
}

// .........................................................
//#define SCALE_POWER SCALE_INT
//#define SCALE_POWER ( SCALE_INT / 10 )
// .........................................................
uint16_t CalcPowerInt( int32_t* testData, PARAMS& params, bool print= false )
{
	std::ofstream* file= NULL;
	if( print )
		file= new std::ofstream( "testout2.txt", std::ios::ate );

	int64_t q0= 0;
	int64_t q1= 0;
	int64_t q2= 0;
	uint64_t range= 0;
	for( auto i= 0; i < params.n_size; ++i )
	{
		q0= ( params.coeff * q1 / SCALE_INT_S ) - q2 + testData[ i ];
		q2= q1;
		q1= q0;
		if( print )
			*file << i << " " << ( q2 ) << " " << ( q0 ) << std::endl;

		if( (uint64_t)abs( q0 ) > range )
			range= abs( q0 );
	}

	if( print )
	{
		file->close( );
		delete file;
	}
	return (uint16_t)( range / 1000 );
}

// .........................................................

//#define CALC_POWER CalcPower
#define CALC_POWER CalcPowerInt

//PrintRange(..)
#define STEP 2
#define RANGE 10


// .........................................................
void PrintRange( double freq )
{
	PARAMS params;
	CreateParams( freq, params );
	int32_t* a= new int32_t[ TEST_DATASET_SIZE ];
	for( double fc= freq - STEP * RANGE; fc < freq + STEP * RANGE; fc+= STEP )
	{
		Generate( fc, params.n_size, a );
		double power= CALC_POWER( a, params, fc == 80.4 );
		std::cout << "freq: " << fc << " \tpower: " << power << std::endl;
	}
	std::cout << std::endl;
	delete [ ] a;
}

// .........................................................
#define TESTING
#ifdef TESTING
// .........................................................

int main( )
{
	std::cout << "New Goertzel\n\n";

	//generate a set of points with noise
	int32_t* a= new int32_t[ TEST_DATASET_SIZE ];
	
	PARAMS params;

	//PrintRange( 77.4 );
	//PrintRange( 254.1 );

	std::cout << std::endl;
	double power= 0.0;
	double freq= 67.0;
	CreateParams( freq, params );
	Generate( freq, params.n_size, a );
	power= CALC_POWER( a, params, true );
	std::cout << "freq: " << freq << " \tpower: " << power << "\n\n";

	freq= 107.2;
	CreateParams( freq, params );
	Generate( freq, params.n_size, a );
	power= CALC_POWER( a, params );
	std::cout << "freq: " << freq << " \tpower: " << power << "\n\n";

	freq= 254.1;
	CreateParams( freq, params );
	Generate( freq, params.n_size, a );
	power= CALC_POWER( a, params );
	std::cout << "freq: " << freq << " \tpower: " << power << "\n\n";

	getchar( );
}

#else // ! TESTING; Generate MSP430 Header

// .........................................................
int main( )
{
	//for 42 bytes it seems to make more sense to
	//scale the power across the band with a factor
	PARAMS params;
	int16_t powerFactor[ PLSS_SIZE ];
	int32_t* a= new int32_t[ TEST_DATASET_SIZE ];
	for( size_t i= 0; i < PLSS_SIZE; ++i )
	{
		double freq= PLs[ i ].freq;
		CreateParams( freq, params );
		Generate( freq, params.n_size, a );
		double power= CALC_POWER( a, params );
		powerFactor[ i ]= (uint16_t)power;
//		std::cout << "freq: " << freq << " \tpower: " << power << "\n";
	}
	getchar( );
	std::ofstream file( "../include/PL.h", std::ios::ate );
	file << "\n/*\npl.h - Used on the target compiler for the MSP430\n";
	file << "This file is generated, DO NO MODIFY HERE!\n*/\n\n;";
	file << "struct PL_PARAMS\n";
	file << "{\n";
	file << "\tuint16_t freq; //the target frequency times 10\n";
	file << "\tuint16_t n_size; //samples in window, adjusted for target frequency\n";
	file << "\tint16_t coeff; //the filtering parameter\n";
	file << "\tuint16_t power_factor; //weight of power, adjust through the band\n";
	file << "};\n\n";

	size_t i= 0;
	//PARAMS params;
	for( ; i < sizeof( PLs ) / sizeof( SourcePL ); ++i )
	{
		CreateParams( PLs[ i ].freq, params );
		file << "static PL_PARAMS PL_" << PLs[ i ].id << "=\n\t{ ";
		file << params.freq << ',' << params.n_size 
			<< ',' << params.coeff << ',' << powerFactor[ i ] << " };\n";
	}
	file << "\n#define PLS_SIZE " << i << "\n";
	file << "#define SCALE_INT " << SCALE_INT_S << "\n";
	file << "#define SAMPLE_RATE " << SAMPLE_RATE_S << "\n";
	file << "\n";
	file.close( );

	getchar( );
	return 0;
}

#endif // TESTING
//New Goertzel

